@@ -73,7 +73,7 @@ void usage(const std::string &cmdname) {
 	printf("  -alpha-overlay                      Generate an RGBA image that can be used as a hillshade mask\n");
 	printf("\n");
 	printf("Shading:\n");
-	printf("  -exag slope_exageration             Exagerate slope (default: %.1f)\n", default_slope_exageration);
+	printf("  -exag slope_exageration             Exaggerate slope (default: %.1f)\n", default_slope_exageration);
 	printf("  -shade ambient diffuse specular_intensity specular_falloff          (default: %.1f, %.1f, %.1f, %.1f)\n",
 		default_shade_params[0], default_shade_params[1], default_shade_params[2], default_shade_params[3]);
 	printf("  -lightvec sun_x sun_y sun_z                                         (default: %.1f, %.1f, %.1f)\n",
@@ -387,9 +387,9 @@ int main(int argc, char *argv[]) {
 			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, &inbuf_prev[0], w, 1, GDT_Float64, 0, 0);
 			GDALRasterIO(src_band, GF_Read, 0, 0, w, 1, &inbuf_this[0], w, 1, GDT_Float64, 0, 0);
 			GDALRasterIO(src_band, GF_Read, 0, 1, w, 1, &inbuf_next[0], w, 1, GDT_Float64, 0, 0);
-			ndv_def.arrayCheckNdv(0, &inbuf_prev[0], &inbuf_ndv_prev[0], w);
-			ndv_def.arrayCheckNdv(0, &inbuf_this[0], &inbuf_ndv_this[0], w);
-			ndv_def.arrayCheckNdv(0, &inbuf_next[0], &inbuf_ndv_next[0], w);
+			ndv_def.getNdvMask(&inbuf_prev[0], GDT_Float64, &inbuf_ndv_prev[0], w);
+			ndv_def.getNdvMask(&inbuf_this[0], GDT_Float64, &inbuf_ndv_this[0], w);
+			ndv_def.getNdvMask(&inbuf_next[0], GDT_Float64, &inbuf_ndv_next[0], w);
 			scale_values(&inbuf_prev[0], w, src_scale, src_offset);
 			scale_values(&inbuf_this[0], w, src_scale, src_offset);
 			scale_values(&inbuf_next[0], w, src_scale, src_offset);
@@ -408,7 +408,7 @@ int main(int argc, char *argv[]) {
 
 			int read_row = (row == h-1) ? row : row+1;
 			GDALRasterIO(src_band, GF_Read, 0, read_row, w, 1, &inbuf_next[0], w, 1, GDT_Float64, 0, 0);
-			ndv_def.arrayCheckNdv(0, &inbuf_next[0], &inbuf_ndv_next[0], w);
+			ndv_def.getNdvMask(&inbuf_next[0], GDT_Float64, &inbuf_ndv_next[0], w);
 			scale_values(&inbuf_next[0], w, src_scale, src_offset);
 		}
 		if(!tex_bands.empty()) {
@@ -676,8 +676,10 @@ void compute_tierow_invaffine(
 	int num_cols, int row, int grid_spacing,
 	double *invaffine_tierow
 ) {
-	double tiecol_left[4];
-	double tiecol_right[4];
+	// this will be initialized on the first iteration, but it is set here to avoid a compiler
+	// warning.
+	double tiecol_left[4] = { 0, 0, 0, 0 };
+	double tiecol_right[4] = { 0, 0, 0, 0 };
 
 	compute_invaffine(georef, 0, row, tiecol_right);
 
