@@ -1,5 +1,5 @@
 /*
-Copyright (c) 2009, Regents of the University of Alaska
+Copyright (c) 2013, Regents of the University of Alaska
 
 All rights reserved.
 
@@ -40,15 +40,15 @@ void usage(const std::string &cmdname) {
 	
 	NdvDef::printUsage();
 
-	printf("\
-\n\
-Misc:\n\
-  -b band_id -b band_id ...   Bands to inspect (default is all bands)\n\
-  -invert              Make mask cover no-data pixels instead of data pixels\n\
-  -erosion             Erode pixels that don't have two consecutive neighbors\n\
-  -v                   Verbose\n\
-\n\
-");
+	printf(
+"\n"
+"Misc:\n"
+"  -b band_id -b band_id ...   Bands to inspect (default is all bands)\n"
+"  -invert              Make mask cover no-data pixels instead of data pixels\n"
+"  -erosion             Erode pixels that don't have two consecutive neighbors\n"
+"  -v                   Verbose\n"
+"\n"
+	);
 	exit(1);
 }
 
@@ -127,6 +127,8 @@ int main(int argc, char **argv) {
 
 	BitGrid mask = get_bitgrid_for_dataset(ds, inspect_bandids, ndv_def, NULL);
 
+	GDALClose(ds);
+
 	if(do_invert) {
 		mask.invert();
 	}
