@@ -26,11 +26,12 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 
 
 
-#include <boost/lexical_cast.hpp>
-#include <boost/foreach.hpp>
 #include <map>
 #include <vector>
 
+#include <boost/lexical_cast.hpp>
+#include <boost/foreach.hpp>
+
 #include "common.h"
 #include "polygon.h"
 #include "polygon-rasterizer.h"
@@ -42,6 +43,7 @@ This code was developed by Dan Stahlke for the Geographic Information Network of
 #include "dp.h"
 #include "excursion_pincher.h"
 #include "beveler.h"
+#include "raster_features.h"
 
 #include <ogrsf_frmts.h>
 #include <cpl_string.h>
@@ -151,11 +153,8 @@ struct GeomOutput {
 		wkt_fh(NULL),
 		wkb_fh(NULL),
 		ogr_ds(NULL),
-		ogr_layer(NULL),
-		class_fld_idx(-1)
-	{
-		for(int i=0; i<4; i++) color_fld_idx[i] = -1;
-	}
+		ogr_layer(NULL)
+	{ }
 
 	CoordSystem out_cs;
 
@@ -169,8 +168,6 @@ struct GeomOutput {
 	std::string ogr_fmt;
 	OGRDataSourceH ogr_ds;
 	OGRLayerH ogr_layer;
-	int class_fld_idx;
-	int color_fld_idx[4];
 };
 
 struct ContainingOption {
@@ -336,7 +333,6 @@ int main(int argc, char **argv) {
 	}
 
 	if(classify) {
-		if(!ndv_def.empty()) fatal_error("-classify option is not compatible with NDV options");
 		if(do_invert) fatal_error("-classify option is not compatible with -invert option");
 		if(mask_out_fn.size()) fatal_error("-classify option is not compatible with -mask-out option");
 	}
@@ -347,15 +343,12 @@ int main(int argc, char **argv) {
 	if(!ds) fatal_error("open failed");
 
 	if(inspect_bandids.empty()) {
-		size_t nbands = classify ? 1 : GDALGetRasterCount(ds);
+		size_t nbands = GDALGetRasterCount(ds);
 		for(size_t i=0; i<nbands; i++) inspect_bandids.push_back(i+1);
 	}
 
-	// FIXME - optional NDV for classify
-	if(!classify) {
-		if(ndv_def.empty()) {
-			ndv_def = NdvDef(ds, inspect_bandids);
-		}
+	if(ndv_def.empty()) {
+		ndv_def = NdvDef(ds, inspect_bandids);
 	}
 
 	CPLPushErrorHandler(CPLQuietErrorHandler);
@@ -378,29 +371,17 @@ int main(int argc, char **argv) {
 			do_pinch_excursions ? PLOT_PINCH : PLOT_CONTOURS);
 	}
 
-	std::vector<uint8_t> raster;
-	BitGrid mask(0, 0);
-	uint8_t usage_array[256];
-	GDALColorTableH color_table = NULL;
+	// only used if classify==true, but doesn't hurt otherwise
+	const FeatureInterpreter feature_interp(ds, inspect_bandids);
+
+	FeatureBitmap *features_bitmap = NULL;
 	if(classify) {
-		if(inspect_bandids.size() != 1) {
-			fatal_error("only one band may be used in classify mode");
-		}
-
-		std::vector<uint8_t> tmp_raster = read_dataset_8bit(ds, inspect_bandids[0], usage_array, dbuf);
-		std::swap(raster, tmp_raster);
-
-		GDALRasterBandH band = GDALGetRasterBand(ds, inspect_bandids[0]);
-		if(GDALGetRasterColorInterpretation(band) == GCI_PaletteIndex) {
-			color_table = GDALGetRasterColorTable(band);
-		}
-	} else {
-		mask = get_bitgrid_for_dataset(ds, inspect_bandids, ndv_def, dbuf);
+		features_bitmap = FeatureBitmap::from_raster(ds, inspect_bandids, ndv_def, dbuf);
 	}
 
 	for(size_t go_idx=0; go_idx<geom_outputs.size(); go_idx++) {
 		GeomOutput &go = geom_outputs[go_idx];
-		
+
 		if(go.wkt_fn.size()) {
 			go.wkt_fh = fopen(go.wkt_fn.c_str(), "w");
 			if(!go.wkt_fh) fatal_error("cannot open output file for WKT");
@@ -416,7 +397,8 @@ int main(int argc, char **argv) {
 			OGRSFDriverH ogr_driver = OGRGetDriverByName(go.ogr_fmt.c_str());
 			if(!ogr_driver) fatal_error("cannot get OGR driver (%s)", go.ogr_fmt.c_str());
 			go.ogr_ds = OGR_Dr_CreateDataSource(ogr_driver, go.ogr_fn.c_str(), NULL);
-			if(!go.ogr_ds) fatal_error("cannot create OGR data source");
+			if(!go.ogr_ds) fatal_error(
+				"cannot create OGR data source (does output file already exist?)");
 
 			std::string layer_name = go.ogr_fn;
 
@@ -432,50 +414,42 @@ int main(int argc, char **argv) {
 			if(!go.ogr_layer) fatal_error("cannot create OGR layer");
 
 			if(classify) {
-				OGRFieldDefnH fld = OGR_Fld_Create("value", OFTInteger);
-				OGR_Fld_SetWidth(fld, 4);
-				OGR_L_CreateField(go.ogr_layer, fld, TRUE);
-				go.class_fld_idx = OGR_FD_GetFieldIndex(OGR_L_GetLayerDefn(go.ogr_layer), "value");
-
-				if(color_table) {
-					const char *names[4] = { "c1", "c2", "c3", "c4" };
-					for(int i=0; i<4; i++) {
-						fld = OGR_Fld_Create(names[i], OFTInteger);
-						OGR_Fld_SetWidth(fld, 4);
-						OGR_L_CreateField(go.ogr_layer, fld, TRUE);
-						go.color_fld_idx[i] = OGR_FD_GetFieldIndex(OGR_L_GetLayerDefn(go.ogr_layer), names[i]);
-					}
-				}
+				feature_interp.create_ogr_fields(go.ogr_layer);
 			}
 		}
 	}
 
 	int num_shapes_written = 0;
 
-	for(int class_id=0; class_id<256; class_id++) {
-		const GDALColorEntry *color = NULL;
-		if(classify) {
-			if(!usage_array[class_id]) continue;
-			printf("\nFeature class %d\n", class_id);
+	std::map<FeatureRawVal, FeatureBitmap::Index> features_list;
+	if(classify) {
+		features_list = features_bitmap->feature_table();
+	} else {
+		// If we are not running in feature classify mode, then just add an arbitrary element
+		// to this list so that the loop below runs once.
+		features_list[FeatureRawVal()];
+	}
 
-			if(color_table) {
-				color = GDALGetColorEntry(color_table, class_id);
-				if(color) printf("  Color=%d,%d,%d,%d\n",
-					color->c1, color->c2, color->c3, color->c4);
+	typedef std::map<FeatureRawVal, FeatureBitmap::Index>::value_type feature_pair_t;
+	size_t feature_idx = 0;
+	BOOST_FOREACH(const feature_pair_t &feature, features_list) {
+		Mpoly feature_poly;
+		{
+			BitGrid mask(0, 0);
+			if(classify) {
+				printf("\nTracing feature %s (%zd of %zd)\n",
+					feature_interp.pixel_to_string(feature.first).c_str(),
+					(++feature_idx), features_list.size());
+				mask = features_bitmap->get_mask_for_feature(feature.second);
+			} else {
+				printf("Reading raster.\n");
+				mask = get_bitgrid_for_dataset(ds, inspect_bandids, ndv_def, dbuf);
 			}
 
-			mask = get_bitgrid_for_8bit_raster(georef.w, georef.h,
-				&raster[0], (uint8_t)class_id);
-		} else {
-			if(class_id != 0) continue;
-		}
+			if(do_invert)  mask.invert();
+			if(do_erosion) mask.erode();
 
-		if(do_invert) {
-			mask.invert();
-		}
-
-		if(do_erosion) {
-			mask.erode();
+			feature_poly = trace_mask(mask, georef.w, georef.h, min_ring_area, trace_no_donuts);
 		}
 
 		if(!containing_options.empty()) {
@@ -499,9 +473,6 @@ int main(int argc, char **argv) {
 			trace_no_donuts = 1;
 		}
 
-		Mpoly feature_poly = trace_mask(mask, georef.w, georef.h, min_ring_area, trace_no_donuts);
-		mask = BitGrid(0, 0); // free some memory
-
 		if(VERBOSE) {
 			size_t num_inner = 0, num_outer = 0, total_pts = 0;
 			for(size_t r_idx=0; r_idx<feature_poly.rings.size(); r_idx++) {
@@ -609,17 +580,11 @@ int main(int argc, char **argv) {
 
 						if(go.ogr_ds) {
 							OGRFeatureH ogr_feat = OGR_F_Create(OGR_L_GetLayerDefn(go.ogr_layer));
-							if(go.class_fld_idx >= 0) OGR_F_SetFieldInteger(ogr_feat, go.class_fld_idx, class_id);
-							if(color) {
-								if(go.color_fld_idx[0] >= 0) OGR_F_SetFieldInteger(
-									ogr_feat, go.color_fld_idx[0], color->c1);
-								if(go.color_fld_idx[1] >= 0) OGR_F_SetFieldInteger(
-									ogr_feat, go.color_fld_idx[1], color->c2);
-								if(go.color_fld_idx[2] >= 0) OGR_F_SetFieldInteger(
-									ogr_feat, go.color_fld_idx[2], color->c3);
-								if(go.color_fld_idx[3] >= 0) OGR_F_SetFieldInteger(
-									ogr_feat, go.color_fld_idx[3], color->c4);
+
+							if(classify) {
+								feature_interp.set_ogr_fields(go.ogr_layer, ogr_feat, feature.first);
 							}
+
 							OGR_F_SetGeometryDirectly(ogr_feat, ogr_geom); // assumes ownership of geom
 							OGR_L_CreateFeature(go.ogr_layer, ogr_feat);
 							OGR_F_Destroy(ogr_feat);
@@ -636,6 +601,8 @@ int main(int argc, char **argv) {
 
 	printf("\n");
 
+	delete(features_bitmap);
+
 	for(size_t go_idx=0; go_idx<geom_outputs.size(); go_idx++) {
 		GeomOutput &go = geom_outputs[go_idx];
 		if(go.wkt_fh) fclose(go.wkt_fh);
